﻿using System.Collections.Generic;

namespace Web.Plugins.Controls.Models
{
    /// <summary>
    /// View model for menu hierarchy container
    /// </summary>
    public class MenuModel
    {
        private readonly List<MenuItem> _menuItems;

        public ICollection<MenuItem> MenuItems
        {
            get
            {
                return _menuItems;
            }
        }

        public MenuModel()
        {
            _menuItems = new List<MenuItem>();
        }

        public void AddMenuItem(MenuItem item)
        {
            _menuItems.Add(item);
        }
    }

    /// <summary>
    /// A single menu item or container of multiple items. Inherits from MenuModel to gain menu item collection
    /// </summary>
    public class MenuItem : MenuModel
    {
        public string Title { get; private set; }
        public string Controller { get; private set; }
        public string Action { get; private set; }

        /// <summary>
        /// Creates menu item for an action link
        /// </summary>
        /// <param name="title"></param>
        /// <param name="controller"></param>
        /// <param name="action"></param>
        public MenuItem(string title, string action, string controller)
            :this(title)
        {
            Controller = controller;
            Action = action;
        }

        /// <summary>
        /// Creates menu item as a container for other items
        /// </summary>
        /// <param name="title"></param>
        public MenuItem(string title)
        {
            Title = title;
        }

        /// <summary>
        /// Returns a System.String that represents the current MenuItem.
        /// </summary>
        /// <returns></returns>
        public override string ToString()
        {
            return string.Format("{0} ({1}/{2}) {3} items", Title, Controller, Action, MenuItems.Count);
        }
    }
}